#ifndef _ANGORA_API_H
#define _ANGORA_API_H

#define ANGORA_SORT_BY_TIME             1
#define ANGORA_SORT_BY_CATEGORY         2

#define ANGORA_TS_BEFORE                1
#define ANGORA_TS_AFTER                 2
#define ANGORA_TS_EQUAL                 4

#define ANGORA_MODE_READER              1
#define ANGORA_MODE_PROVIDER            2
#define ANGORA_MODE_ACCESS_VERIFY       4

 /*
  * XXX Need to put #define's here to represent all the
  * ACL types in the database.
  */

typedef char Boolean;

typedef struct AngoraSubscription {
     char *path;
     Boolean positive;
     Boolean override;
     Boolean class;
} AngoraSubscription;

typedef struct AngoraSubscriptionArray {
     int size;
     AngoraSubscription *subscriptions;
}
     
typedef time_t AngoraTimeStamp;

typedef struct AngoraState {
     char *path;
     AngoraTimeStamp timestamp;
} AngoraState;

typedef struct AngoraStateArray {
     int size;
     AngoraState *states;
} AngoraStateArray;

/* XXX opaque -- fill in later */

typedef struct AngoraReaderCategory AngoraReaderCategory;
typedef struct AngoraProviderCategory AngoraProviderCategory;

typedef struct AngoraCategory {
     AngoraReaderCategory reader;
     AngoraProviderCategory provider;
} AngoraCategory;

typedef struct AngoraCategoryArray {
     int size;
     AngoraCategory *categories;
} AngoraCategoryArray;

typedef struct AngoraReaderClass AngoraReaderClass;
typedef struct AngoraProviderClass AngoraProviderClass;

typedef struct AngoraClass {
     AngoraReaderClass reader;
     AngoraProviderClass provider;
} AngoraClass;

typedef struct AngoraClassArray {
     int size;
     AngoraClass *classes;
} AngoraClassArray;

typedef struct AngoraReaderMessage AngoraReaderMessage;
typedef struct AngoraProviderMessage AngoraProviderMessage;

typedef struct AngoraMessage {
     AngoraReaderMessage reader;
     AngoraProviderMessage provider;
} AngoraMessage;

typedef AngoraTimeStamp AngoraMessageId;

typedef struct AngoraMessageIdArray {
     int size;
     AngoraMessageId *ids;
} AngoraMessageIdArray;

/* This is intentionally opaque */

typedef void *AngoraQuery;

/* XXX not sure this should be opaque */

typedef void *AngoraPrincipal;

typedef union AngoraAclEntry {
     int type;
     Boolean positive;
     struct {
          int type;
          Boolean positive;
          AngoraPrincipal principal;
     } principal;
     struct {
          int type;
          Boolean positive;
          char *path;
          int which;
     } acl;
} AngoraAclEntry;

typedef struct AngoraAcl {
     int size;
     AngoraAclEntry *entries;
} AngoraAcl;

     
/* XXX Need functions for constructing principals */

int     AngoraInitialize();
void    AngoraDeinitialize();
void    AngoraMode(
            int *mode                               /* in/out */
        );

int     AngoraSubParse(
            char *input_string,
            AngoraSubscription **subscription       /* out */
        );
int     AngoraSubUnparse(
            AngoraSubscription *subscription,
            char **output_string                    /* out */
        );

int     AngoraSubAdd(
            AngoraSubscription *subscription,
            AngoraSubscriptionArray *subscriptions  /* in/out */
        );
int     AngoraSubDelete(
            int subscription_number,
            AngoraSubscriptionArray *subscriptions  /* in/out */
        );
int     AngoraSubMerge(
            AngoraSubscriptionArray *subscriptions1,
            AngoraSubscriptionArray *subscriptions2,
            AngoraSubscriptionArray **merged_subs,  /* out */
        );
int     AngoraSubGetDefaults(
            AngoraSubscriptionArray **subscriptions /* out */
        );

int     AngoraSubRead(
            char *file,
            AngoraSubscriptionArray **subscriptions /* out */
        );
int     AngoraSubWrite(
            char *file
            AngoraSubscriptionArray *subscriptions
        );

int     AngoraSubMatch(
            AngoraSubscription *subscription,
            char *path,
            int *matches                            /* out */
        );
int     AngoraSubMatchArray(
            AngoraSubscriptionArray *subscriptions,
            char *path,
            int *would_read                         /* out */
        );

void    AngoraSubFree(
            AngoraSubscription *subscription
        );
int     AngoraSubArrayFree(
            AngoraSubscriptionArray *subscriptions
        );
void    AngoraStringFree(
            char *string
        );

int     AngoraStateRead(
            char *file,
            AngoraStateArray **states               /* out */
        );
int     AngoraStateWrite(
            char *file,
            AngoraStateArray *states
        );
int     AngoraStateFind(
            AngoraStateArray *states,
            char *path,
            AngoraTimeStamp **timestamp             /* out */
        );
int     AngoraStateUpdate(
            AngoraCategory *category,
            AngoraTimeStamp *timestamp,
            AngoraStateArray **states               /* in/out */
        );
int     AngoraStatePrune(
            AngoraStateCategory **states            /* in/out */
        );
void    AngoraStateArrayFree(
            AngoraStateArray *states
        );

int     AngoraGetClass(
            char *path,
            AngoraClass **class                     /* in/out */
        );
int     AngoraGetCategory(
            char *path,
            AngoraCategory **category               /* in/out */
        );
int     AngoraGetSubClasses(
            AngoraClass class,
            AngoraClassArray **sub_classes          /* out */
        );
int     AngoraGetSubCategories(
            AngoraCategory category,
            AngoraCategoryArray **sub_categories    /* out */
        );

int     AngoraCreateClass(
            char *path
        );
int     AngoraCreateCategory(
            char *path
        );
int     AngoraDeleteClass(
            char *path,
            Boolean recursive
        );
int     AngoraDeleteCategory(
            char *path,
            Boolean remove_messages
        );
int     AngoraRenameNode(
            char *old_path,
            char *new_path
        );
int     AngoraModifyClass(
            char *path,
            AngoraClass *class_data
        );
int     AngoraModifyCategory(
            char *path,
            AngoraClass *class_data
        );

void    AngoraClassFree(
            AngoraClass *class
        );
void    AngoraCategoryFree(
            AngoraCategory *category
        );
void    AngoraClassArrayFree(
            AngoraClassArray *classes
        );
void    AngoraCategoryArrayFree(
            AngoraCategoryArray *categories
        );

int     AngoraGetNewMessages(
            AngoraSubscriptionArray *subscriptions,
            AngoraStateArray *state,
            AngoraMessageIdArray **messages         /* out */
        );
int     AngoraSortMessages(
            int how,
            AngoraMessageIdArray **messages         /* in/out */
        );
int     AngoraGetNextMessage(
            AngoraMessageId current,
            AngoraMessageId *next                   /* out */
        );
int     AngoraGetPreviousMessage(
            AngoraMessageId current,
            AngoraMessageId *previous               /* out */
        );

int     AngoraQueryCategory(
            char *path,
            AngoraQuery *query                      /* out */
        );
int     AngoraQueryClass(
            char *path,
            AngoraQuery *query                      /* out */
        );
int     AngoraQueryTimeStamp(
            AngoraTimeStamp stamp,
            int range,
            AngoraQuery *query                      /* out */
        );
int	AngoraQuerySubscription(
            AngoraSubscription *subscription,
            AngoraQuery *query                      /* out */
        );
int     AngoraQueryAuthor(
            AngoraPrincipal principal,
            AngoraQuery *query                      /* out */
        );
int     AngoraQueryText(
            char *string,
            AngoraQuery *query                      /* out */
        );

int     AngoraQueryNegation(
            AngoraQuery in,
            AngoraQuery *out                        /* out */
        );
int     AngoraQueryUnion(
            AngoraQuery in1,
            AngoraQuery in2,
            AngoraQuery *out                        /* out */
        );
int     AngoraQueryIntersection(
            AngoraQuery in1,
            AngoraQuery in2,
            AngoraQuery *out                        /* out */
        );
int     AngoraQueryExecute(
            AngoraQuery query,
            AngoraMessageIdArray *messages          /* out */
        );
void    AngoraQueryFree(
            AngoraQuery query
        );

void    AngoraMessageIdArrayFree(
            AngoraMessageIdArray *ids
        );

int     AngoraGetMessage(
            AngoraMessageId id,
            AngoraMessage **message                 /* out */
        );
void    AngoraMessageFree(
            AngoraMessage *message
        );

int     AngoraCreateMessage(
            AngoraMessage *message,
            AngoraMessageId *id                     /* out */
        );
int     AngoraExpireMessage(
            AngoraMessageId id
        );
int     AngoraDeleteMessage(
            AngoraMessageId id
        );
int     AngoraReplaceMessage(
            AngoraMessageId id,
            AngoraMessage *message,
            AngoraMessageId *new_id                 /* out */
        );

int     AngoraCreateKerberosPrincipal(
            char *name,
            char *instance,
            char *realm,
            AngoraPrincipal *principal              /* out */
        );
int     AngoraCreateGlobalPrincipal(
            AngoraPrincipal *principal              /* out */
        );
int     AngoraPrincipalFree(
            AngoraPrincipal principal
        );

int     AngoraGetAcl(
            char *path,
            int type,
            AngoraAcl **acl                         /* out */
        );
int     AngoraAddToAcl(
            char *path,
            int type,
            AngoraAclEntry *entry,
            AngoraAcl **acl                         /* in/out */
        );
int     AngoraRemoveFromAcl(
            char *path,
            int type,
            AngoraAclEntry *entry,
            AngoraAcl **acl                         /* in/out */
        );
int     AngoraSetAcl(
            char *path,
            int type,
            AngoraAcl *acl
        );

int     AngoraDisplayXawInitialize(
            Widget top_level
        );
int     AngoraDisplayXawMessage(
            AngoraMessage *message
        );
void    AngoraDisplayXawDeinitialize();

int     AngoraDisplayXmInitialize(
            Widget top_level
        );
int     AngoraDisplayXmMessage(
            AngoraMessage *message
        );
void    AngoraDisplayXmDeinitialize();

int     AngoraDisplayTtyInitialize(
            int in_fd,
            int out_fd
        );
int     AngoraDisplayTtyMessage(
            AngoraMessage *message
        );
void    AngoraDisplayTtyDeinitialize();

#endif /* _ANGORA_API_H */
